//
//  BLEDeviceManager.h
//
//

/* NotificationNames in this Class, which can be called by NSNotificationCenter.
 
 * Notification01
 *      Name:
 *                  @"AceBLEMgrEventNotice"
 *      userInfo:
 *                  @"uuid"
 *                  @"AceBLEMgrEventID"                         // AceBLEMgrEvent
 
 * Notification02
 *      Name:
 *                  @"updateRealTimeValue_basedata"
 *      userInfo:
 *                  @"uuid"
 *                  @"data"                                     // Real-Time weight value (basedata: the original weight data from Scale)
 
 * Notification03
 *      Name:
 *                  @"updateRealTimeValue_finalValue_basedata"
 *      userInfo:
 *                  @"uuid"
 *                  @"data"                                     // Real-Time weight final value (basedata: the original weight data from Scale)
 *                  @"ownerId"                                  // ownerId of weight record. 
 *                                                              // if ownerId=0(default), means unkonwn;
 *                                                              // if ownerId>0, means on [[BLEDeviceManager sharedInstance] startMeasuringForUser:(int)userId]
 *                  @"timestamp"                                // timestamp from BLE scale.
 
 * Notification04
 *      Name:
 *                  @"updateRealTimeValue"
 *      userInfo:
 *                  @"uuid"
 *                  @"data"                                     // Real-Time weight value (kg or lb)
                                                                // depend on [BLEDeviceManager sharedInstance] setUnit:(AceWeightUnit)unit]
                                                                // kg = [[BLEDeviceManager sharedInstance] displayValueByKG:basedata]
                                                                // lb = [[BLEDeviceManager sharedInstance] displayValueByLB:basedata]
 *                  @"ownerId"                                  // ownerId of weight record.
 *                                                              // if ownerId=0(default), means unkonwn;
 *                                                              // if ownerId>0, means on [[BLEDeviceManager sharedInstance] startMeasuringForUser:(int)userId]
 *                  @"timestamp"                                // timestamp from BLE scale.
 
 * Notification05
 *      Name:
 *                  @"updateRealTimeValue_finalValue"
 *      userInfo:
 *                  @"uuid"
 *                  @"data"                                     // result weight value (kg or lb)
                                                                // depend on [BLEDeviceManager sharedInstance] setUnit:(AceWeightUnit)unit]
                                                                // kg = [[BLEDeviceManager sharedInstance] displayValueByKG:basedata]
                                                                // lb = [[BLEDeviceManager sharedInstance] displayValueByLB:basedata]
 
 * Notification06
 *      Name:
 *                  @"updateDonwloadProgress"
 *      userInfo:
 *                  @"uuid"
 *                  @"counter"                                  // the number of new records that app has received
 *                  @"total"                                    // the number of all new records
 
 * Notification07
 *      Name:
 *                  @"updateRecordDownloadFromScale_basedata"
 *      userInfo:
 *                  @"uuid"
 *                  @"data"                                     // weight value if record (basedata: the original weight data from Scale)
 *                  @"timestamp"                                // timestamp from BLE scale.
 
 *
 *
 */



#import <CoreBluetooth/CoreBluetooth.h>
#import "config.h"
#import "BLEDeviceManagerStorageDelegate.h"
#import "BLEDeviceManagerRealTimeWeighDelegate.h"

@interface BLEDeviceManager : NSObject <CBCentralManagerDelegate, CBPeripheralDelegate>
{
}

@property (nonatomic, assign) AceBLEMgrPattern patternType;
@property (strong, nonatomic) NSMutableArray *tempPeripherals;      // for scan
@property (strong, nonatomic) NSMutableArray *peripherals;          // for reuse
@property (strong, nonatomic) CBCentralManager *CM; 
@property (strong, nonatomic) CBPeripheral *activePeripheral;

@property (nonatomic, assign) id <BLEDeviceManagerStorageDelegate> storrageDelegate;
@property (nonatomic, assign) id <BLEDeviceManagerRealTimeWeighDelegate> realTimeWeighDelegate;

#pragma mark -
+ (BLEDeviceManager *)sharedInstance;                      // Same as [BLEDeviceManager sharedInstanceWithPatternType:PatternAllFeature]
+ (BLEDeviceManager *)sharedInstanceWithPatternType:(AceBLEMgrPattern)pattern;


#pragma mark -
- (NSString*)getSDKVertion;
- (void)setAutoFetchDataOnConnectted:(BOOL)yesOrNo;         // set the swith of auto fetch data on connected. (default NO)

#pragma mark -
- (int)scanNewDevice:(int)timeout;                          // start scan BLE device (only Pally scale), within "timeout" seconds.
- (void)stopScan;                                           // stop scan
//- (BOOL)requestRemoteData;                                // Removed from Ver 1.0
- (BOOL)requestRemoteDataWithTimestamp:(NSTimeInterval)second_start;    // download new (newer than second_start) weight record those stored in BLE device.
- (void)disconnectDevice;
//- (void)clearLocalData;                                   // Removed from Ver 1.0
- (BOOL)isConnected;                                        // return a BOOL value of whether a BLE device is correctly connected. 
- (NSString*)nameOfConnectedBLE;                            // get name of connected BLE device
- (void)weightScale_turnOn:(BOOL)yesOrNo;                   // turn on notify : real-time weight value (Scale will light, if be setted to "ON"）
- (void)weightScale_resultValue_switchOn:(BOOL)yesOrNo;     // turn on notify : result weight value
- (void)clearRemoteRecords;                                 // send comand to BLE device (clear all weight record stored in BLE device)
- (void)resetRemoteBondInfo;                                // send comand to BLE device (clear all bonding infomation stored in BLE device)
- (void)setTimeCalibration;                                 // send comand to BLE device (set time of BLE device, with the current timestump of iOS)
- (void)connectPeripheral:(CBPeripheral *)peripheral;       // try to connect a peripheral (BLE device)

#pragma mark - shortcut

#pragma mark - 000000
- (void)retrieveKnownPeripheral_withDeviceUUID:(NSString *)uuidStr;     // try to reconnect a peripheral(BLE device) by its CFUUIDRef string
//- (void)retrieveKnownPeripherals;                         // Removed from Ver 1.0

#pragma mark -
- (void)startMeasuringForUser:(int)userId;                  // Forced to add record to userId，Temporarily disable owner determine
- (void)stopMeasuring;                                      // Enable owner determine


#pragma mark - APIs for get Device Information
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_SystemID"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_SystemID;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_ModelNumber"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_ModelNumber;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_SerialNumber"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_SerialNumber;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_FirmwareRevision"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_FirmwareRevision;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_HardwareRevision"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_HardwareRevision;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_SoftwareRevision"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_SoftwareRevision;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_ManufacturerName"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_ManufacturerName;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_IEEE_11073_20601_Regulatory_Certification_Data_List"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_IEEE_11073_20601_Regulatory_Certification_Data_List;
/*
 * return       YES:    Means BLE device valid, and send SDK has sent read command to BLE device.
 *                      You can get a NSData value by a NSNotification named "DeviceInfo_PnPID"
 *              NO:     Means BLE device noe valid.
 */
- (BOOL)getDeviceInfo_PnPID;


#pragma mark - Helper
- (void)setUnit:(AceWeightUnit)unit;                        // set the Weight record unit of "updateRealTimeValue" and "updateRealTimeValue_finalValue" returned from NSNotificationCenter.
- (int)getUnit;                                             // get the current setting for unit.
- (float)displayValueByLB:(float)valueFromScale;            // format valueFromScale(basedata: the original weight data from Scale) to LB
- (float)displayValueByKG:(float)valueFromScale;            // format valueFromScale(basedata: the original weight data from Scale) to KG
- (float)databaseValue_fromKG:(float)kgValue;               // format KG value to basedata(the original weight data from Scale)
- (float)databaseValue_fromLB:(float)lbValue;               // format LB value to basedata(the original weight data from Scale)



@end